//
//  cadrePhoto.java
//  detectionContours
//
//  Created by Thomas Jannaud on 22/12/09.
//  Copyright 2009 Jannaud's Technical Company. All rights reserved.
//

import java.awt.Graphics;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import java.awt.Color;
import java.awt.event.*;
import java.util.LinkedList;
import java.util.Collection;
import java.awt.Rectangle;

public class cadrePhoto extends JLabel implements MouseListener, MouseMotionListener {
	private int lastX = -1, lastY = -1, newX = -1, newY = -1;
	private LinkedList<myPoint> selection = new LinkedList<myPoint>();
	private LinkedList<myPoint> last_selection = new LinkedList<myPoint>();
	private int offsetY = 0; // if the image is smaller than the JLabel
	private myGreyPicture image;
	private static final int RECTANGLE = 0, FREE = 1;
	private int selection_mode = FREE, detectionMode = contourDetector.GRAPHE_MIN_CUT;
	private showMe show_me;
	private contourDetector detectorOfContours;
	private JLabel lblGreyLevel;
	private boolean inColor = false;
	
	public cadrePhoto(JLabel lblGrey) {
		super("No picture yet");
		addMouseMotionListener(this);
		addMouseListener(this);
		image = new myGreyPicture();
		lblGreyLevel = lblGrey;
 	}
	
	public void setPicture(String filename) {
		System.out.println("open " + filename);
		image.initWith(filename);
		publish();
	}
	
	public void setImageInColor(boolean a) {
		inColor = a;
		publish();
	}
	
	public void publish() {
		if (image.isGood()) {
			if (inColor)
				setIcon(new ImageIcon(image.getColorImage()));
			else
				setIcon(new ImageIcon(image.getGreyImage()));
			setText("");
		}
		else
			setText("No picture yet");
		repaint();
	}
	
	public void reload() {
		if (image.isGood())
			image.reload();
		publish();
	}
	
	public void stop() {
		if (detectorOfContours != null)
			detectorOfContours.finished();
	}
	
	public void setDetectionMode(int a) {
		detectionMode = a;
		int last_selection_mode = selection_mode;
		selection_mode = (detectionMode == contourDetector.GRAPHE_MIN_CUT) ? FREE : RECTANGLE;
		if (selection_mode != last_selection_mode)
			deselect();
	}
	
	public void detectContours(int thickness) {
		reload();
		if (image.isGood()) {
			detectorOfContours = new contourDetector(image, detectionMode, mode_color == GREY, show_me, thickness);
			Object o = null;
			if (detectionMode == contourDetector.GRADIENT_CANNY && isSelection())
				o = new Rectangle(Math.min(lastX, newX), Math.min(lastY, newY), Math.abs(newX - lastX), Math.abs(newY - lastY));
			else if (detectionMode == contourDetector.GRAPHE_MIN_CUT && isSelection())
				o = selection;
			o = selection;
			detectorOfContours.setSelection(o);
			show_me = new showMe(this);
			show_me.start();
			detectorOfContours.start();
		}
	}
	
	public void deselect() {
		lastX = -1; lastY = -1; newX = -1; newY = -1;
		selection.clear();
		repaint();
	}
	
	public static final int GREY = 0, RED = 1;
	private int mode_color = GREY;
	
	public void setGreyMode(int mode) {
		mode_color = mode;
	}
	
	/* -----------------------------
	 GRAPHIC INTERFACE
	 ----------------------------- */
	
	public void paint(Graphics g) {
		super.paint(g);
		g.setColor(Color.blue);
		if (image.isGood())
			offsetY = Math.max(0, (getHeight() - image.getHeight()) / 2);
		g.setColor(Color.blue);
		if (isSelection() && image.isGood()) {
			if (selection_mode == RECTANGLE)
				g.drawRect(Math.min(lastX, newX), offsetY + Math.min(lastY, newY), Math.abs(newX - lastX), Math.abs(newY - lastY));
			else {
				for(myPoint p : selection)
					g.drawRect(p.x, offsetY + p.y, 0, 0);
				g.drawLine(selection.getLast().x, offsetY + selection.getLast().y, selection.getFirst().x, offsetY + selection.getFirst().y);
			}
		}
	}
	
	private boolean isSelection() {
		if (selection_mode == RECTANGLE)
			return (lastX >= 0 && lastY >= 0 && newX >= 0 && newY >= 0);
		else
			return !selection.isEmpty();
	}
	
	public void mousePressed(MouseEvent e) {
		if (image.isGood()) {
			last_selection.clear();
			for(myPoint p : selection)
				last_selection.addLast(new myPoint(p));
			selection.clear();
			lastX = Math.min(e.getX(), image.getWidth());
			lastY = Math.min(Math.max(-1, e.getY() - offsetY), image.getHeight());
			selection.add(new myPoint(lastX, lastY, myPoint.CONTOURS));
		}
	}
	
	public void mouseDragged(MouseEvent e) {
		if (image.isGood()) {
			newX = Math.min(e.getX(), image.getWidth());
			newY = Math.min(Math.max(0, e.getY() - offsetY), image.getHeight());
			myPoint p = selection.removeLast();
			addLine(selection, p, new myPoint(newX, newY, myPoint.CONTOURS));
			repaint();
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		if (image.isGood() && selection_mode == FREE) {
			if (selection.size() < 15) {
				selection.clear();
				for(myPoint p : last_selection)
					selection.addLast(new myPoint(p));
			}
			complete(selection);
			repaint();
		}
	}
	
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {
		if (image.isGood()) {
			String greyLevel = "";
			int x = e.getX(), y = e.getY() - offsetY;
			float f = image.displayedGreyComponent(x, y);
			if (f >= 0.f)
				greyLevel = "(" + x + ", " + y + ") : " + f;
			else
				greyLevel = "";
			lblGreyLevel.setText(greyLevel);
		}
	}
	
	// ------------------
	// graphic primitives
	// ------------------
	
	private void addLine(Collection<myPoint> c, final myPoint p1, final myPoint p2) {
		// be careful, this adds p1 and p2
		if (Math.abs(p2.x - p1.x) > Math.abs(p2.y - p1.y)) {
			final int inc_x = (p1.x < p2.x) ? 1 : -1;
			final int delta = Math.abs(p2.x - p1.x);
			final int m = p2.y - p1.y;
			final int inc_y = 2 * Math.abs(m);
			final int dy = m > 0 ? 1 : -1;
			int y = p1.y;
			int s0 = -delta;
			final int inc_s0 = -2 * delta;
			for(int i = p1.x; i != p2.x; i += inc_x) {
				c.add(new myPoint(i, y, myPoint.CONTOURS));
				s0 += inc_y;
				if (s0 > 0) {
					y += dy;
					s0 += inc_s0;
				}
			}
		}
		else {
			final int inc_y = (p1.y < p2.y) ? 1 : -1;
			final int delta = Math.abs(p2.y - p1.y);
			final int m = p2.x - p1.x;
			final int inc_x = 2 * Math.abs(m);
			final int dx = m > 0 ? 1 : -1;
			int x = p1.x;
			int s0 = -delta;
			final int inc_s0 = -2 * delta;
			for(int i = p1.y; i != p2.y; i += inc_y) {
				c.add(new myPoint(x, i, myPoint.CONTOURS));
				s0 += inc_x;
				if (s0 > 0) {
					x += dx;
					s0 += inc_s0;
				}
			}
		}
		c.add(p2);
	}
	
	private void complete(LinkedList<myPoint> c) {
		if (c.size() > 2) {
			myPoint p1 = c.removeLast(), p2 = c.removeFirst();
			addLine(c, p1, p2);
		}
		else
			c.clear();
	}
}